@@ -17,7 +17,7 @@ bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
   else
 #endif
   {
-    if (NewMhd.Flags & MHD_COMMENT)
+    if ((NewMhd.Flags & MHD_COMMENT)!=0)
     {
       // Old style (RAR 2.9) archive comment embedded into the main 
       // archive header.
@@ -43,7 +43,7 @@ bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
 #endif
   }
 #ifndef SFX_MODULE
-  if (OldFormat && (OldMhd.Flags & MHD_PACK_COMMENT) || !OldFormat && CommHead.Method!=0x30)
+  if (OldFormat && (OldMhd.Flags & MHD_PACK_COMMENT)!=0 || !OldFormat && CommHead.Method!=0x30)
   {
     if (!OldFormat && (CommHead.UnpVer < 15 || CommHead.UnpVer > UNP_VER || CommHead.Method > 0x35))
       return(false);
@@ -54,7 +54,7 @@ bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
     uint UnpCmtLength;
     if (OldFormat)
     {
-#ifdef NOCRYPT
+#ifdef RAR_NOCRYPT
       return(false);
 #else
       UnpCmtLength=GetByte();
@@ -110,8 +110,12 @@ bool Archive::GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW)
     if (CmtDataW!=NULL)
     {
       CmtDataW->Alloc(CmtSize+1);
-      CmtData->Push(0);
-      CharToWide(DataA,CmtDataW->Addr(),CmtSize+1);
+
+      // It can cause reallocation, so we should not use 'DataA' variable
+      // with previosuly saved CmtData->Addr() after Push() call.
+      CmtData->Push(0); 
+
+      CharToWide((char *)CmtData->Addr(),CmtDataW->Addr(),CmtSize+1);
       CmtData->Alloc(CmtSize);
       CmtDataW->Alloc(wcslen(CmtDataW->Addr()));
     }
