@@ -20,6 +20,7 @@ Archive::Archive(RAROptions *InitCmd)
   LatestTime.Reset();
   Protected=false;
   Encrypted=false;
+  FailedHeaderDecryption=false;
   BrokenFileHeader=false;
   LastReadBlock=0;
 
@@ -36,7 +37,7 @@ Archive::Archive(RAROptions *InitCmd)
   VolWrite=0;
   AddingFilesSize=0;
   AddingHeadersSize=0;
-#if !defined(SHELL_EXT) && !defined(NOCRYPT)
+#if !defined(SHELL_EXT) && !defined(RAR_NOCRYPT)
   *HeadersSalt=0;
   *SubDataSalt=0;
 #endif
