@@ -1,6 +1,8 @@
 #ifndef _UNRAR_DLL_
 #define _UNRAR_DLL_
 
+#pragma pack(1)
+
 #define ERAR_END_ARCHIVE        10
 #define ERAR_NO_MEMORY          11
 #define ERAR_BAD_DATA           12
@@ -15,8 +17,9 @@
 #define ERAR_UNKNOWN            21
 #define ERAR_MISSING_PASSWORD   22
 
-#define RAR_OM_LIST           0
-#define RAR_OM_EXTRACT        1
+#define RAR_OM_LIST              0
+#define RAR_OM_EXTRACT           1
+#define RAR_OM_LIST_INCSPLIT     2
 
 #define RAR_SKIP              0
 #define RAR_TEST              1
@@ -25,13 +28,14 @@
 #define RAR_VOL_ASK           0
 #define RAR_VOL_NOTIFY        1
 
-#define RAR_DLL_VERSION       4
+#define RAR_DLL_VERSION       5
 
 #ifdef _UNIX
 #define CALLBACK
 #define PASCAL
 #define LONG long
 #define HANDLE void *
+#define LPARAM long
 #define UINT unsigned int
 #endif
 
@@ -91,26 +95,28 @@ struct RAROpenArchiveData
   unsigned int CmtState;
 };
 
+typedef int (CALLBACK *UNRARCALLBACK)(UINT msg,LPARAM UserData,LPARAM P1,LPARAM P2);
+
 struct RAROpenArchiveDataEx
 {
   char         *ArcName;
   wchar_t      *ArcNameW;
-  unsigned int OpenMode;
-  unsigned int OpenResult;
+  unsigned int  OpenMode;
+  unsigned int  OpenResult;
   char         *CmtBuf;
-  unsigned int CmtBufSize;
-  unsigned int CmtSize;
-  unsigned int CmtState;
-  unsigned int Flags;
-  unsigned int Reserved[32];
+  unsigned int  CmtBufSize;
+  unsigned int  CmtSize;
+  unsigned int  CmtState;
+  unsigned int  Flags;
+  UNRARCALLBACK Callback;
+  LPARAM        UserData;
+  unsigned int  Reserved[28];
 };
 
 enum UNRARCALLBACK_MESSAGES {
   UCM_CHANGEVOLUME,UCM_PROCESSDATA,UCM_NEEDPASSWORD
 };
 
-typedef int (CALLBACK *UNRARCALLBACK)(UINT msg,LONG UserData,LONG P1,LONG P2);
-
 typedef int (PASCAL *CHANGEVOLPROC)(char *ArcName,int Mode);
 typedef int (PASCAL *PROCESSDATAPROC)(unsigned char *Addr,int Size);
 
@@ -125,7 +131,7 @@ int    PASCAL RARReadHeader(HANDLE hArcData,struct RARHeaderData *HeaderData);
 int    PASCAL RARReadHeaderEx(HANDLE hArcData,struct RARHeaderDataEx *HeaderData);
 int    PASCAL RARProcessFile(HANDLE hArcData,int Operation,char *DestPath,char *DestName);
 int    PASCAL RARProcessFileW(HANDLE hArcData,int Operation,wchar_t *DestPath,wchar_t *DestName);
-void   PASCAL RARSetCallback(HANDLE hArcData,UNRARCALLBACK Callback,LONG UserData);
+void   PASCAL RARSetCallback(HANDLE hArcData,UNRARCALLBACK Callback,LPARAM UserData);
 void   PASCAL RARSetChangeVolProc(HANDLE hArcData,CHANGEVOLPROC ChangeVolProc);
 void   PASCAL RARSetProcessDataProc(HANDLE hArcData,PROCESSDATAPROC ProcessDataProc);
 void   PASCAL RARSetPassword(HANDLE hArcData,char *Password);
@@ -135,4 +141,6 @@ int    PASCAL RARGetDllVersion();
 }
 #endif
 
+#pragma pack()
+
 #endif
