@@ -26,8 +26,15 @@
 #define  AV_VER                 20
 #define  PROTECT_VER            20
 
+
 #define  MHD_VOLUME         0x0001U
+
+// Old style main archive comment embed into main archive header. Must not
+// be used in new archives anymore. Must never be used with MHD_ENCRYPTVER
+// or other flags changing the size of main header. RAR expects the fixed
+// size of main header preceding the comment if MHD_COMMENT is found.
 #define  MHD_COMMENT        0x0002U
+
 #define  MHD_LOCK           0x0004U
 #define  MHD_SOLID          0x0008U
 #define  MHD_PACK_COMMENT   0x0010U
@@ -41,9 +48,14 @@
 #define  LHD_SPLIT_BEFORE   0x0001U
 #define  LHD_SPLIT_AFTER    0x0002U
 #define  LHD_PASSWORD       0x0004U
+
+// Old style file comment embed into file header. Must not be used
+// in new archives anymore.
 #define  LHD_COMMENT        0x0008U
+
 #define  LHD_SOLID          0x0010U
 
+
 #define  LHD_WINDOWMASK     0x00e0U
 #define  LHD_WINDOW64       0x0000U
 #define  LHD_WINDOW128      0x0020U
@@ -59,15 +71,15 @@
 #define  LHD_SALT           0x0400U
 #define  LHD_VERSION        0x0800U
 #define  LHD_EXTTIME        0x1000U
-#define  LHD_EXTFLAGS       0x2000U
+#define  LHD_EXTAREA        0x2000U
 
 #define  SKIP_IF_UNKNOWN    0x4000U
 #define  LONG_BLOCK         0x8000U
 
-#define  EARC_NEXT_VOLUME   0x0001U // not last volume
-#define  EARC_DATACRC       0x0002U // store CRC32 of RAR archive (now used only in volumes)
-#define  EARC_REVSPACE      0x0004U // reserve space for end of REV file 7 byte record
-#define  EARC_VOLNUMBER     0x0008U // store a number of current volume
+#define  EARC_NEXT_VOLUME   0x0001U // Not last volume.
+#define  EARC_DATACRC       0x0002U // Store CRC32 of RAR archive (now is used only in volumes).
+#define  EARC_REVSPACE      0x0004U // Reserve space for end of REV file 7 byte record.
+#define  EARC_VOLNUMBER     0x0008U // Store a number of current volume.
 
 enum HEADER_TYPE {
   MARK_HEAD=0x72,MAIN_HEAD=0x73,FILE_HEAD=0x74,COMM_HEAD=0x75,AV_HEAD=0x76,
@@ -129,7 +141,7 @@ struct MarkHeader
 struct BaseBlock
 {
   ushort HeadCRC;
-  HEADER_TYPE HeadType;//byte
+  HEADER_TYPE HeadType; // 1 byte.
   ushort Flags;
   ushort HeadSize;
 
@@ -218,8 +230,15 @@ struct FileHeader:BlockHeader
 
 struct EndArcHeader:BaseBlock
 {
-  uint ArcDataCRC;  // optional archive CRC32
-  ushort VolNumber; // optional current volume number
+  // Optional CRC32 of entire archive up to start of EndArcHeader block.
+  // Present if EARC_DATACRC flag is set.
+  uint ArcDataCRC;  
+  
+  // Optional number of current volume.
+  // Present if EARC_VOLNUMBER flag is set.
+  ushort VolNumber; 
+
+  // 7 additional zero bytes can be stored here if EARC_REVSPACE is set.
 };
 
 
