@@ -55,9 +55,10 @@ bool WideToChar(const wchar *Src,char *Dest,size_t DestSize)
 
   // We tried to return the zero terminated string if conversion is failed,
   // but it does not work well. WideCharToMultiByte returns 'failed' code
-  // even if we wanted to convert only a part of string and passed DestSize
-  // smaller than required for fully converted string. Such call is the valid
-  // behavior in RAR code and we do not expect the empty string in this case.
+  // and partially converted string even if we wanted to convert only a part
+  // of string and passed DestSize smaller than required for fully converted
+  // string. Such call is the valid behavior in RAR code and we do not expect
+  // the empty string in this case.
 
   return(RetCode);
 }
@@ -121,6 +122,7 @@ bool CharToWide(const char *Src,wchar *Dest,size_t DestSize)
 }
 
 
+// SrcSize is in wide characters, not in bytes.
 byte* WideToRaw(const wchar *Src,byte *Dest,size_t SrcSize)
 {
   for (size_t I=0;I<SrcSize;I++,Src++)
@@ -178,7 +180,8 @@ void WideToUtf(const wchar *Src,char *Dest,size_t DestSize)
 }
 
 
-void UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
+// Dest can be NULL if we only need to check validity of Src.
+bool UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
 {
   long dsize=(long)DestSize;
   dsize--;
@@ -213,19 +216,25 @@ void UtfToWide(const char *Src,wchar *Dest,size_t DestSize)
           }
           else
             break;
-    if (--dsize<0)
+    if (Dest!=NULL && --dsize<0)
       break;
     if (d>0xffff)
     {
-      if (--dsize<0 || d>0x10ffff)
+      if (Dest!=NULL && --dsize<0 || d>0x10ffff)
         break;
-      *(Dest++)=((d-0x10000)>>10)+0xd800;
-      *(Dest++)=(d&0x3ff)+0xdc00;
+      if (Dest!=NULL)
+      {
+        *(Dest++)=((d-0x10000)>>10)+0xd800;
+        *(Dest++)=(d&0x3ff)+0xdc00;
+      }
     }
     else
-      *(Dest++)=d;
+      if (Dest!=NULL)
+        *(Dest++)=d;
   }
-  *Dest=0;
+  if (Dest!=NULL)
+    *Dest=0;
+  return *Src==0; // Return success if we reached the end of source string.
 }
 
 
@@ -295,6 +304,16 @@ int toupperw(int ch)
 }
 
 
+int tolowerw(int ch)
+{
+#ifdef _WIN_ALL
+  return((int)(LPARAM)CharLowerW((wchar *)(uint)ch));
+#else
+  return((ch<128) ? loctolower(ch):ch);
+#endif
+}
+
+
 int atoiw(const wchar *s)
 {
   int n=0;
@@ -328,7 +347,10 @@ void SupportDBCS::Init()
 
 char* SupportDBCS::charnext(const char *s)
 {
-  return (char *)(IsLeadByte[(byte)*s] ? s+2:s+1);
+  // Zero cannot be the trail byte. So if next byte after the lead byte
+  // is 0, the string is corrupt and we'll better return the pointer to 0,
+  // to break string processing loops.
+  return (char *)(IsLeadByte[(byte)*s] && s[1]!=0 ? s+2:s+1);
 }
 
 
